;
; BatchNum - by DednDave - Version 1
;
; A Simple Batch File Utility:
; Originally intended for use in assembler batch files, this utility has a number of other uses.
; By using any number of small text files, extremely complex batch processes can be formed.
;
;   I) opens the text file specified on the command line (full drive:\path\filename.ext accepted)
;     1) place ONLY the file drive:\path\filename.ext on the command line
;        extraneous characters will invalidate the filename string
;        this does not apply to standard DOS redirection characters
;        i.e. BatchNum C:\test.txt>nul will work, but the display text is redirected to the NUL device
;     2) if the filename is invalid or the file does not exist, exits with an error message
;        also returns an exit code of 0 (any invalid condition returns 0)
;
;  II) reads up to 8192 bytes from the file
;
; III) searches for the first occurance of the string "BATCH" (must be all capital letters)
;     1) if the string is not found, an exit code value of 0 is returned
;
;  IV) parses the numeric value that follows and returns it as an exit code
;     1) non-numeric characters between elements are ignored (except end of line or end of file)
;     2) after the first numeric digit is found, any non-numeric character terminates parsing
;     3) maximum value is 255
;
;  V) displays the value:
;     BATCHNUM: nnn
;
; All of the following text file examples return an exit code of 255:
;
; BATCH255
; BATCH 255
; BATCH=255
; BATCH = 255
; BATCH Value = 255
; BATCHNUM: 255
; BATCH 1024
;
; Resluting display text:
;
; BATCHNUM: 255
;
; As mentioned above, BatchNum C:\test.txt>nul can be used to silence the displayed value text.
;
;-------------------------------------------------------------------------------------------------------

        INCLUDE \masm32\include\masm32rt.inc

;-------------------------------------------------------------------------------------------------------

;display text used for testing

        ECHO    BATCH 200

;-------------------------------------------------------------------------------------------------------

        .CONST

ErrMsg1 db 'Invalid Filename',0
ErrMsg2 db 'File Not Found',0
ErrMsg3 db 'Error Reading Input File',0

        .DATA?

FSize   dd ?,?
FBuff   db 8193 dup(?)

;-------------------------------------------------------------------------------------------------------

        .CODE

_main   PROC

;read the command line to get the path\name of the scratch file

        INVOKE  GetCommandLine
        cld
        xchg    eax,esi
        mov     edx,offset ErrMsg1           ;'Invalid Filename'
        xor     eax,eax

PCL000: lodsb
        cmp     al,20h
        jz      PCL002

        or      al,al
        jnz     PCL000

PCL001: push    edx
        print   chr$(13,10,'BatchNum: ')
        pop     eax
        print   eax
        print   chr$(13,10)
        jmp     BExit0

PCL002: lodsb
        or      al,al
        jz      PCL001

        cmp     al,20h
        jbe     PCL002

        dec     esi

;open the file and get the size

        INVOKE  CreateFile,
                esi,
                GENERIC_READ,
                FILE_SHARE_READ,
                NULL,
                OPEN_EXISTING,
                FILE_ATTRIBUTE_NORMAL,
                NULL
        inc     eax                          ;error ?
        mov     edx,offset ErrMsg2           ;'File Not Found'
        jz      PCL001                       ;yes - error exit

        dec     eax
        push    eax                          ;handle on the stack
        INVOKE  GetFileSizeEx,eax,offset FSize
        or      eax,eax
        jnz     FRead0

FError: pop     eax
        INVOKE  CloseHandle,eax
        mov     edx,offset ErrMsg3           ;'Error Reading Input File'
        jmp     PCL001

;read the first 8 Kb of the file into the buffer and close it

FRead0: xor     eax,eax
        mov     ecx,8192
        cmp     eax,FSize[4]                 ;test the high dword
        jnz     FRead1

        mov     eax,FSize
        cmp     eax,ecx                      ;test the low dword
        ja      FRead1

        xchg    eax,ecx                      ;ECX = bytes to read

FRead1: mov     eax,[esp]                    ;EAX = file handle
        INVOKE  ReadFile,
                eax,
                offset FBuff,
                ecx,
                offset FSize,
                NULL
        or      eax,eax
        jz      FError

        pop     eax
        INVOKE  CloseHandle,eax

;search for the first occurance of the string "BATCH"

        push    42h                          ;'B'
        mov     edx,'HCTA'                   ;EDX = 'ATCH'
        pop     eax                          ;EAX = 'B'
        mov     edi,offset FBuff             ;EDI = buffer address
        mov     ecx,FSize                    ;ECX = bytes read

BSrch0: cmp     ecx,5
        jbe     BExit0

        repnz   scasb
        jnz     BExit0

        cmp     edx,[edi]
        jnz     BSrch0

        add     edi,4
        sub     ecx,4
        jbe     BExit0

;parse the numeric value (0-255)

Parse0: mov     al,[edi]
        cmp     al,39h
        ja      Parse1

        cmp     al,30h
        jae     Parse2

        cmp     al,0Dh
        jz      BExit0

        cmp     al,0Ah
        jz      BExit0

Parse1: inc     edi
        dec     ecx
        jnz     Parse0

        jmp short BExit0

Parse2: mov     esi,edi
        inc     edi
        dec     ecx
        jz      Parse4

        mov     al,3
        cmp     eax,ecx
        jae     Parse3

        mov     ecx,eax

Parse3: mov     al,[edi]
        cmp     al,39h
        ja      Parse4

        cmp     al,30h
        jb      Parse4

        inc     edi
        dec     ecx
        jnz     Parse3

Parse4: mov     [edi],ah
        mov     eax,uval(esi)
        mov     edx,255
        cmp     eax,edx
        jbe     BExit1

        xchg    eax,edx
        jmp short BExit1

;return the value in the exit code - if the string is not found, return 0

BExit0: xor     eax,eax

BExit1: push    eax
        print   chr$('BATCHNUM: ')
        mov     eax,[esp]
        print   ustr$(eax),13,10
        pop     eax
        INVOKE  ExitProcess,eax

_main   ENDP

;-------------------------------------------------------------------------------------------------------

        END     _main

